@@ -13,9 +13,9 @@ use futures::{
 };
 use fuzzy::StringMatchCandidate;
 use gpui::{
-    actions, point, size, AnyElement, AppContext, BackgroundExecutor, Bounds, DevicePixels,
-    EventEmitter, Global, PromptLevel, ReadGlobal, Subscription, Task, TitlebarOptions,
-    UpdateGlobal, View, WindowBounds, WindowHandle, WindowOptions,
+    actions, percentage, point, size, Animation, AnimationExt, AppContext, BackgroundExecutor,
+    Bounds, EventEmitter, Global, PromptLevel, ReadGlobal, Subscription, Task, TitlebarOptions,
+    Transformation, UpdateGlobal, View, WindowBounds, WindowHandle, WindowOptions,
 };
 use heed::{types::SerdeBincode, Database, RoTxn};
 use language::{language_settings::SoftWrap, Buffer, LanguageRegistry};
@@ -23,15 +23,18 @@ use parking_lot::RwLock;
 use picker::{Picker, PickerDelegate};
 use rope::Rope;
 use serde::{Deserialize, Serialize};
+use settings::Settings;
 use std::{
+    cmp::Reverse,
     future::Future,
     path::PathBuf,
     sync::{atomic::AtomicBool, Arc},
     time::Duration,
 };
+use theme::ThemeSettings;
 use ui::{
-    div, prelude::*, IconButtonShape, ListHeader, ListItem, ListItemSpacing, ListSubHeader,
-    ParentElement, Render, SharedString, Styled, TitleBar, Tooltip, ViewContext, VisualContext,
+    div, prelude::*, IconButtonShape, ListItem, ListItemSpacing, ParentElement, Render,
+    SharedString, Styled, TitleBar, Tooltip, ViewContext, VisualContext,
 };
 use util::{paths::PROMPTS_DIR, ResultExt, TryFutureExt};
 use uuid::Uuid;
@@ -77,11 +80,7 @@ pub fn open_prompt_library(
         cx.spawn(|cx| async move {
             let store = store.await?;
             cx.update(|cx| {
-                let bounds = Bounds::centered(
-                    None,
-                    size(DevicePixels::from(1024), DevicePixels::from(768)),
-                    cx,
-                );
+                let bounds = Bounds::centered(None, size(px(1024.0), px(768.0)), cx);
                 cx.open_window(
                     WindowOptions {
                         titlebar: Some(TitlebarOptions {
@@ -94,7 +93,7 @@ pub fn open_prompt_library(
                     },
                     |cx| cx.new_view(|cx| PromptLibrary::new(store, language_registry, cx)),
                 )
-            })
+            })?
         })
     }
 }
@@ -121,41 +120,23 @@ struct PromptEditor {
 struct PromptPickerDelegate {
     store: Arc<PromptStore>,
     selected_index: usize,
-    entries: Vec<PromptPickerEntry>,
+    matches: Vec<PromptMetadata>,
 }
 
 enum PromptPickerEvent {
-    Selected { prompt_id: Option<PromptId> },
+    Selected { prompt_id: PromptId },
     Confirmed { prompt_id: PromptId },
     Deleted { prompt_id: PromptId },
     ToggledDefault { prompt_id: PromptId },
 }
 
-#[derive(Debug)]
-enum PromptPickerEntry {
-    DefaultPromptsHeader,
-    DefaultPromptsEmpty,
-    AllPromptsHeader,
-    AllPromptsEmpty,
-    Prompt(PromptMetadata),
-}
-
-impl PromptPickerEntry {
-    fn prompt_id(&self) -> Option<PromptId> {
-        match self {
-            PromptPickerEntry::Prompt(metadata) => Some(metadata.id),
-            _ => None,
-        }
-    }
-}
-
 impl EventEmitter<PromptPickerEvent> for Picker<PromptPickerDelegate> {}
 
 impl PickerDelegate for PromptPickerDelegate {
-    type ListItem = AnyElement;
+    type ListItem = ListItem;
 
     fn match_count(&self) -> usize {
-        self.entries.len()
+        self.matches.len()
     }
 
     fn selected_index(&self) -> usize {
@@ -164,14 +145,11 @@ impl PickerDelegate for PromptPickerDelegate {
 
     fn set_selected_index(&mut self, ix: usize, cx: &mut ViewContext<Picker<Self>>) {
         self.selected_index = ix;
-        let prompt_id = if let Some(PromptPickerEntry::Prompt(prompt)) =
-            self.entries.get(self.selected_index)
-        {
-            Some(prompt.id)
-        } else {
-            None
-        };
-        cx.emit(PromptPickerEvent::Selected { prompt_id });
+        if let Some(prompt) = self.matches.get(self.selected_index) {
+            cx.emit(PromptPickerEvent::Selected {
+                prompt_id: prompt.id,
+            });
+        }
     }
 
     fn placeholder_text(&self, _cx: &mut WindowContext) -> Arc<str> {
@@ -180,48 +158,24 @@ impl PickerDelegate for PromptPickerDelegate {
 
     fn update_matches(&mut self, query: String, cx: &mut ViewContext<Picker<Self>>) -> Task<()> {
         let search = self.store.search(query);
-        let prev_prompt_id = self
-            .entries
-            .get(self.selected_index)
-            .and_then(|mat| mat.prompt_id());
+        let prev_prompt_id = self.matches.get(self.selected_index).map(|mat| mat.id);
         cx.spawn(|this, mut cx| async move {
-            let (entries, selected_index) = cx
+            let (matches, selected_index) = cx
                 .background_executor()
                 .spawn(async move {
-                    let prompts = search.await;
-                    let (default_prompts, prompts) = prompts
-                        .into_iter()
-                        .partition::<Vec<_>, _>(|prompt| prompt.default);
-
-                    let mut entries = Vec::new();
-                    entries.push(PromptPickerEntry::DefaultPromptsHeader);
-                    if default_prompts.is_empty() {
-                        entries.push(PromptPickerEntry::DefaultPromptsEmpty);
-                    } else {
-                        entries.extend(default_prompts.into_iter().map(PromptPickerEntry::Prompt));
-                    }
-
-                    entries.push(PromptPickerEntry::AllPromptsHeader);
-                    if prompts.is_empty() {
-                        entries.push(PromptPickerEntry::AllPromptsEmpty);
-                    } else {
-                        entries.extend(prompts.into_iter().map(PromptPickerEntry::Prompt));
-                    }
+                    let matches = search.await;
 
                     let selected_index = prev_prompt_id
                         .and_then(|prev_prompt_id| {
-                            entries
-                                .iter()
-                                .position(|entry| entry.prompt_id() == Some(prev_prompt_id))
+                            matches.iter().position(|entry| entry.id == prev_prompt_id)
                         })
-                        .or_else(|| entries.iter().position(|entry| entry.prompt_id().is_some()))
                         .unwrap_or(0);
-                    (entries, selected_index)
+                    (matches, selected_index)
                 })
                 .await;
 
             this.update(&mut cx, |this, cx| {
-                this.delegate.entries = entries;
+                this.delegate.matches = matches;
                 this.delegate.set_selected_index(selected_index, cx);
                 cx.notify();
             })
@@ -230,7 +184,7 @@ impl PickerDelegate for PromptPickerDelegate {
     }
 
     fn confirm(&mut self, _secondary: bool, cx: &mut ViewContext<Picker<Self>>) {
-        if let Some(PromptPickerEntry::Prompt(prompt)) = self.entries.get(self.selected_index) {
+        if let Some(prompt) = self.matches.get(self.selected_index) {
             cx.emit(PromptPickerEvent::Confirmed {
                 prompt_id: prompt.id,
             });
@@ -245,81 +199,73 @@ impl PickerDelegate for PromptPickerDelegate {
         selected: bool,
         cx: &mut ViewContext<Picker<Self>>,
     ) -> Option<Self::ListItem> {
-        let prompt = self.entries.get(ix)?;
-        let element = match prompt {
-            PromptPickerEntry::DefaultPromptsHeader => ListHeader::new("Default Prompts")
-                .inset(true)
-                .start_slot(Icon::new(IconName::ZedAssistant))
-                .selected(selected)
-                .into_any_element(),
-            PromptPickerEntry::DefaultPromptsEmpty => {
-                ListSubHeader::new("Star a prompt to add it to the default context")
-                    .inset(true)
-                    .selected(selected)
-                    .into_any_element()
-            }
-            PromptPickerEntry::AllPromptsHeader => ListHeader::new("All Prompts")
-                .inset(true)
-                .start_slot(Icon::new(IconName::Library))
-                .selected(selected)
-                .into_any_element(),
-            PromptPickerEntry::AllPromptsEmpty => ListSubHeader::new("No prompts")
-                .inset(true)
-                .selected(selected)
-                .into_any_element(),
-            PromptPickerEntry::Prompt(prompt) => {
-                let default = prompt.default;
-                let prompt_id = prompt.id;
-                ListItem::new(ix)
-                    .inset(true)
-                    .spacing(ListItemSpacing::Sparse)
-                    .selected(selected)
-                    .child(Label::new(
-                        prompt.title.clone().unwrap_or("Untitled".into()),
-                    ))
-                    .end_hover_slot(
-                        h_flex()
-                            .gap_2()
-                            .child(
-                                IconButton::new("delete-prompt", IconName::Trash)
-                                    .shape(IconButtonShape::Square)
-                                    .tooltip(move |cx| Tooltip::text("Delete Prompt", cx))
-                                    .on_click(cx.listener(move |_, _, cx| {
-                                        cx.emit(PromptPickerEvent::Deleted { prompt_id })
-                                    })),
-                            )
-                            .child(
-                                IconButton::new(
-                                    "toggle-default-prompt",
-                                    if default {
-                                        IconName::ZedAssistantFilled
-                                    } else {
-                                        IconName::ZedAssistant
-                                    },
-                                )
-                                .shape(IconButtonShape::Square)
-                                .tooltip(move |cx| {
-                                    Tooltip::text(
-                                        if default {
-                                            "Remove from Default Prompt"
-                                        } else {
-                                            "Add to Default Prompt"
-                                        },
-                                        cx,
-                                    )
-                                })
-                                .on_click(cx.listener(
-                                    move |_, _, cx| {
-                                        cx.emit(PromptPickerEvent::ToggledDefault { prompt_id })
-                                    },
-                                )),
-                            ),
+        let prompt = self.matches.get(ix)?;
+        let default = prompt.default;
+        let prompt_id = prompt.id;
+        let element = ListItem::new(ix)
+            .inset(true)
+            .spacing(ListItemSpacing::Sparse)
+            .selected(selected)
+            .child(h_flex().h_5().line_height(relative(1.)).child(Label::new(
+                prompt.title.clone().unwrap_or("Untitled".into()),
+            )))
+            .end_slot::<IconButton>(default.then(|| {
+                IconButton::new("toggle-default-prompt", IconName::SparkleFilled)
+                    .selected(true)
+                    .icon_color(Color::Accent)
+                    .shape(IconButtonShape::Square)
+                    .tooltip(move |cx| Tooltip::text("Remove from Default Prompt", cx))
+                    .on_click(cx.listener(move |_, _, cx| {
+                        cx.emit(PromptPickerEvent::ToggledDefault { prompt_id })
+                    }))
+            }))
+            .end_hover_slot(
+                h_flex()
+                    .gap_2()
+                    .child(
+                        IconButton::new("delete-prompt", IconName::Trash)
+                            .icon_color(Color::Muted)
+                            .shape(IconButtonShape::Square)
+                            .tooltip(move |cx| Tooltip::text("Delete Prompt", cx))
+                            .on_click(cx.listener(move |_, _, cx| {
+                                cx.emit(PromptPickerEvent::Deleted { prompt_id })
+                            })),
                     )
-                    .into_any_element()
-            }
-        };
+                    .child(
+                        IconButton::new("toggle-default-prompt", IconName::Sparkle)
+                            .selected(default)
+                            .selected_icon(IconName::SparkleFilled)
+                            .icon_color(if default { Color::Accent } else { Color::Muted })
+                            .shape(IconButtonShape::Square)
+                            .tooltip(move |cx| {
+                                Tooltip::text(
+                                    if default {
+                                        "Remove from Default Prompt"
+                                    } else {
+                                        "Add to Default Prompt"
+                                    },
+                                    cx,
+                                )
+                            })
+                            .on_click(cx.listener(move |_, _, cx| {
+                                cx.emit(PromptPickerEvent::ToggledDefault { prompt_id })
+                            })),
+                    ),
+            );
         Some(element)
     }
+
+    fn render_editor(&self, editor: &View<Editor>, cx: &mut ViewContext<Picker<Self>>) -> Div {
+        h_flex()
+            .bg(cx.theme().colors().editor_background)
+            .rounded_md()
+            .overflow_hidden()
+            .flex_none()
+            .py_1()
+            .px_2()
+            .mx_2()
+            .child(editor.clone())
+    }
 }
 
 impl PromptLibrary {
@@ -331,11 +277,13 @@ impl PromptLibrary {
         let delegate = PromptPickerDelegate {
             store: store.clone(),
             selected_index: 0,
-            entries: Vec::new(),
+            matches: Vec::new(),
         };
 
         let picker = cx.new_view(|cx| {
-            let picker = Picker::list(delegate, cx).modal(false).max_height(None);
+            let picker = Picker::uniform_list(delegate, cx)
+                .modal(false)
+                .max_height(None);
             picker.focus(cx);
             picker
         });
@@ -358,11 +306,7 @@ impl PromptLibrary {
     ) {
         match event {
             PromptPickerEvent::Selected { prompt_id } => {
-                if let Some(prompt_id) = *prompt_id {
-                    self.load_prompt(prompt_id, false, cx);
-                } else {
-                    self.focus_picker(&Default::default(), cx);
-                }
+                self.load_prompt(*prompt_id, false, cx);
             }
             PromptPickerEvent::Confirmed { prompt_id } => {
                 self.load_prompt(*prompt_id, true, cx);
@@ -549,21 +493,23 @@ impl PromptLibrary {
             if let Some(prompt_id) = prompt_id {
                 if picker
                     .delegate
-                    .entries
+                    .matches
                     .get(picker.delegate.selected_index())
                     .map_or(true, |old_selected_prompt| {
-                        old_selected_prompt.prompt_id() != Some(prompt_id)
+                        old_selected_prompt.id != prompt_id
                     })
                 {
                     if let Some(ix) = picker
                         .delegate
-                        .entries
+                        .matches
                         .iter()
-                        .position(|mat| mat.prompt_id() == Some(prompt_id))
+                        .position(|mat| mat.id == prompt_id)
                     {
                         picker.set_selected_index(ix, true, cx);
                     }
                 }
+            } else {
+                picker.focus(cx);
             }
         });
         cx.notify();
@@ -746,14 +692,13 @@ impl PromptLibrary {
             .child(
                 h_flex()
                     .p(Spacing::Small.rems(cx))
-                    .border_b_1()
-                    .border_color(cx.theme().colors().border)
                     .h(TitleBar::height(cx))
                     .w_full()
                     .flex_none()
                     .justify_end()
                     .child(
                         IconButton::new("new-prompt", IconName::Plus)
+                            .style(ButtonStyle::Transparent)
                             .shape(IconButtonShape::Square)
                             .tooltip(move |cx| Tooltip::for_action("New Prompt", &NewPrompt, cx))
                             .on_click(|_, cx| {
@@ -775,12 +720,21 @@ impl PromptLibrary {
             .flex_none()
             .min_w_64()
             .children(self.active_prompt_id.and_then(|prompt_id| {
+                let buffer_font = ThemeSettings::get_global(cx).buffer_font.family.clone();
                 let prompt_metadata = self.store.metadata(prompt_id)?;
                 let prompt_editor = &self.prompt_editors[&prompt_id];
+                let focus_handle = prompt_editor.editor.focus_handle(cx);
+                let current_model = CompletionProvider::global(cx).model();
+                let token_count = prompt_editor.token_count.map(|count| count.to_string());
+
                 Some(
                     h_flex()
+                        .id("prompt-editor-inner")
                         .size_full()
                         .items_start()
+                        .on_click(cx.listener(move |_, _, cx| {
+                            cx.focus(&focus_handle);
+                        }))
                         .child(
                             div()
                                 .on_action(cx.listener(Self::focus_picker))
@@ -788,8 +742,8 @@ impl PromptLibrary {
                                 .on_action(cx.listener(Self::cancel_last_inline_assist))
                                 .flex_grow()
                                 .h_full()
-                                .pt(Spacing::Large.rems(cx))
-                                .pl(Spacing::Large.rems(cx))
+                                .pt(Spacing::XXLarge.rems(cx))
+                                .pl(Spacing::XXLarge.rems(cx))
                                 .child(prompt_editor.editor.clone()),
                         )
                         .child(
@@ -797,49 +751,92 @@ impl PromptLibrary {
                                 .w_12()
                                 .py(Spacing::Large.rems(cx))
                                 .justify_start()
-                                .items_center()
-                                .gap_4()
-                                .child(
-                                    IconButton::new(
-                                        "toggle-default-prompt",
-                                        if prompt_metadata.default {
-                                            IconName::ZedAssistantFilled
-                                        } else {
-                                            IconName::ZedAssistant
-                                        },
-                                    )
-                                    .size(ButtonSize::Large)
-                                    .shape(IconButtonShape::Square)
-                                    .tooltip(move |cx| {
-                                        Tooltip::for_action(
-                                            if prompt_metadata.default {
-                                                "Remove from Default Prompt"
-                                            } else {
-                                                "Add to Default Prompt"
-                                            },
-                                            &ToggleDefaultPrompt,
-                                            cx,
+                                .items_end()
+                                .gap_1()
+                                .child(h_flex().h_8().font_family(buffer_font).when_some_else(
+                                    token_count,
+                                    |tokens_ready, token_count| {
+                                        tokens_ready.pr_3().justify_end().child(
+                                            // This isn't actually a button, it just let's us easily add
+                                            // a tooltip to the token count.
+                                            Button::new("token_count", token_count.clone())
+                                                .style(ButtonStyle::Transparent)
+                                                .color(Color::Muted)
+                                                .tooltip(move |cx| {
+                                                    Tooltip::with_meta(
+                                                        format!("{} tokens", token_count,),
+                                                        None,
+                                                        format!(
+                                                            "Model: {}",
+                                                            current_model.display_name()
+                                                        ),
+                                                        cx,
+                                                    )
+                                                }),
                                         )
-                                    })
-                                    .on_click(|_, cx| {
-                                        cx.dispatch_action(Box::new(ToggleDefaultPrompt));
-                                    }),
+                                    },
+                                    |tokens_loading| {
+                                        tokens_loading.w_12().justify_center().child(
+                                            Icon::new(IconName::ArrowCircle)
+                                                .size(IconSize::Small)
+                                                .color(Color::Muted)
+                                                .with_animation(
+                                                    "arrow-circle",
+                                                    Animation::new(Duration::from_secs(4)).repeat(),
+                                                    |icon, delta| {
+                                                        icon.transform(Transformation::rotate(
+                                                            percentage(delta),
+                                                        ))
+                                                    },
+                                                ),
+                                        )
+                                    },
+                                ))
+                                .child(
+                                    h_flex().justify_center().w_12().h_8().child(
+                                        IconButton::new("toggle-default-prompt", IconName::Sparkle)
+                                            .style(ButtonStyle::Transparent)
+                                            .selected(prompt_metadata.default)
+                                            .selected_icon(IconName::SparkleFilled)
+                                            .icon_color(if prompt_metadata.default {
+                                                Color::Accent
+                                            } else {
+                                                Color::Muted
+                                            })
+                                            .shape(IconButtonShape::Square)
+                                            .tooltip(move |cx| {
+                                                Tooltip::text(
+                                                    if prompt_metadata.default {
+                                                        "Remove from Default Prompt"
+                                                    } else {
+                                                        "Add to Default Prompt"
+                                                    },
+                                                    cx,
+                                                )
+                                            })
+                                            .on_click(|_, cx| {
+                                                cx.dispatch_action(Box::new(ToggleDefaultPrompt));
+                                            }),
+                                    ),
                                 )
                                 .child(
-                                    IconButton::new("delete-prompt", IconName::Trash)
-                                        .shape(IconButtonShape::Square)
-                                        .tooltip(move |cx| {
-                                            Tooltip::for_action("Delete Prompt", &DeletePrompt, cx)
-                                        })
-                                        .on_click(|_, cx| {
-                                            cx.dispatch_action(Box::new(DeletePrompt));
-                                        }),
-                                )
-                                .children(prompt_editor.token_count.map(|token_count| {
-                                    h_flex()
-                                        .justify_center()
-                                        .child(Label::new(token_count.to_string()))
-                                })),
+                                    h_flex().justify_center().w_12().h_8().child(
+                                        IconButton::new("delete-prompt", IconName::Trash)
+                                            .size(ButtonSize::Large)
+                                            .style(ButtonStyle::Transparent)
+                                            .shape(IconButtonShape::Square)
+                                            .tooltip(move |cx| {
+                                                Tooltip::for_action(
+                                                    "Delete Prompt",
+                                                    &DeletePrompt,
+                                                    cx,
+                                                )
+                                            })
+                                            .on_click(|_, cx| {
+                                                cx.dispatch_action(Box::new(DeletePrompt));
+                                            }),
+                                    ),
+                                ),
                         ),
                 )
             }))
@@ -848,6 +845,14 @@ impl PromptLibrary {
 
 impl Render for PromptLibrary {
     fn render(&mut self, cx: &mut ViewContext<Self>) -> impl IntoElement {
+        let (ui_font, ui_font_size) = {
+            let theme_settings = ThemeSettings::get_global(cx);
+            (theme_settings.ui_font.clone(), theme_settings.ui_font_size)
+        };
+
+        let theme = cx.theme().clone();
+        cx.set_rem_size(ui_font_size);
+
         h_flex()
             .id("prompt-manager")
             .key_context("PromptLibrary")
@@ -858,6 +863,8 @@ impl Render for PromptLibrary {
             }))
             .size_full()
             .overflow_hidden()
+            .font(ui_font)
+            .text_color(theme.colors().text)
             .child(self.render_prompt_list(cx))
             .child(self.render_active_prompt(cx))
     }
@@ -1026,7 +1033,7 @@ impl PromptStore {
         let cached_metadata = self.metadata_cache.read().metadata.clone();
         let executor = self.executor.clone();
         self.executor.spawn(async move {
-            if query.is_empty() {
+            let mut matches = if query.is_empty() {
                 cached_metadata
             } else {
                 let candidates = cached_metadata
@@ -1052,7 +1059,9 @@ impl PromptStore {
                     .into_iter()
                     .map(|mat| cached_metadata[mat.candidate_id].clone())
                     .collect()
-            }
+            };
+            matches.sort_by_key(|metadata| Reverse(metadata.default));
+            matches
         })
     }
 
