@@ -1,4 +1,4 @@
-use crate::assistant_panel::ConversationEditor;
+use crate::assistant_panel::ContextEditor;
 use anyhow::Result;
 pub use assistant_slash_command::{SlashCommand, SlashCommandOutput, SlashCommandRegistry};
 use editor::{CompletionProvider, Editor};
@@ -20,6 +20,7 @@ pub mod active_command;
 pub mod default_command;
 pub mod fetch_command;
 pub mod file_command;
+pub mod now_command;
 pub mod project_command;
 pub mod prompt_command;
 pub mod rustdoc_command;
@@ -29,7 +30,7 @@ pub mod tabs_command;
 pub(crate) struct SlashCommandCompletionProvider {
     commands: Arc<SlashCommandRegistry>,
     cancel_flag: Mutex<Arc<AtomicBool>>,
-    editor: Option<WeakView<ConversationEditor>>,
+    editor: Option<WeakView<ContextEditor>>,
     workspace: Option<WeakView<Workspace>>,
 }
 
@@ -43,7 +44,7 @@ pub(crate) struct SlashCommandLine {
 impl SlashCommandCompletionProvider {
     pub fn new(
         commands: Arc<SlashCommandRegistry>,
-        editor: Option<WeakView<ConversationEditor>>,
+        editor: Option<WeakView<ContextEditor>>,
         workspace: Option<WeakView<Workspace>>,
     ) -> Self {
         Self {
