@@ -52,8 +52,14 @@ use multi_buffer::{
     ToOffset, ToPoint,
 };
 use serde::Deserialize;
-use std::ops::Add;
-use std::{any::TypeId, borrow::Cow, fmt::Debug, num::NonZeroU32, ops::Range, sync::Arc};
+use std::{
+    any::TypeId,
+    borrow::Cow,
+    fmt::Debug,
+    num::NonZeroU32,
+    ops::{Add, Range, Sub},
+    sync::Arc,
+};
 use sum_tree::{Bias, TreeMap};
 use tab_map::{TabMap, TabSnapshot};
 use text::LineIndent;
@@ -1027,6 +1033,14 @@ impl Add for DisplayRow {
     }
 }
 
+impl Sub for DisplayRow {
+    type Output = Self;
+
+    fn sub(self, other: Self) -> Self::Output {
+        DisplayRow(self.0 - other.0)
+    }
+}
+
 impl DisplayPoint {
     pub fn new(row: DisplayRow, column: u32) -> Self {
         Self(BlockPoint(Point::new(row.0, column)))
