@@ -1,13 +1,13 @@
 //! Provides conversion from rustdoc's HTML output to Markdown.
 
-#![deny(missing_docs)]
-
 mod html_element;
-mod markdown;
+pub mod markdown;
 mod markdown_writer;
-mod structure;
+pub mod structure;
 
+use std::cell::RefCell;
 use std::io::Read;
+use std::rc::Rc;
 
 use anyhow::{Context, Result};
 use html5ever::driver::ParseOpts;
@@ -16,25 +16,18 @@ use html5ever::tendril::TendrilSink;
 use html5ever::tree_builder::TreeBuilderOpts;
 use markup5ever_rcdom::RcDom;
 
-use crate::markdown::{HeadingHandler, ListHandler, ParagraphHandler, StyledTextHandler};
-use crate::markdown_writer::{HandleTag, MarkdownWriter};
+use crate::markdown::{
+    HeadingHandler, ListHandler, ParagraphHandler, StyledTextHandler, TableHandler,
+};
+use crate::markdown_writer::MarkdownWriter;
+
+pub use crate::markdown_writer::{HandleTag, TagHandler};
+use crate::structure::rustdoc::RustdocItem;
 
 /// Converts the provided HTML to Markdown.
-pub fn convert_html_to_markdown(html: impl Read) -> Result<String> {
+pub fn convert_html_to_markdown(html: impl Read, handlers: &mut Vec<TagHandler>) -> Result<String> {
     let dom = parse_html(html).context("failed to parse HTML")?;
 
-    let handlers: Vec<Box<dyn HandleTag>> = vec![
-        Box::new(ParagraphHandler),
-        Box::new(HeadingHandler),
-        Box::new(ListHandler),
-        Box::new(StyledTextHandler),
-        Box::new(structure::rustdoc::RustdocChromeRemover),
-        Box::new(structure::rustdoc::RustdocHeadingHandler),
-        Box::new(structure::rustdoc::RustdocCodeHandler),
-        Box::new(structure::rustdoc::RustdocTableHandler::new()),
-        Box::new(structure::rustdoc::RustdocItemHandler),
-    ];
-
     let markdown_writer = MarkdownWriter::new();
     let markdown = markdown_writer
         .run(&dom.document, handlers)
@@ -44,27 +37,32 @@ pub fn convert_html_to_markdown(html: impl Read) -> Result<String> {
 }
 
 /// Converts the provided rustdoc HTML to Markdown.
-pub fn convert_rustdoc_to_markdown(html: impl Read) -> Result<String> {
-    let dom = parse_html(html).context("failed to parse rustdoc HTML")?;
+pub fn convert_rustdoc_to_markdown(html: impl Read) -> Result<(String, Vec<RustdocItem>)> {
+    let item_collector = Rc::new(RefCell::new(structure::rustdoc::RustdocItemCollector::new()));
 
-    let handlers: Vec<Box<dyn HandleTag>> = vec![
-        Box::new(ParagraphHandler),
-        Box::new(HeadingHandler),
-        Box::new(ListHandler),
-        Box::new(StyledTextHandler),
-        Box::new(structure::rustdoc::RustdocChromeRemover),
-        Box::new(structure::rustdoc::RustdocHeadingHandler),
-        Box::new(structure::rustdoc::RustdocCodeHandler),
-        Box::new(structure::rustdoc::RustdocTableHandler::new()),
-        Box::new(structure::rustdoc::RustdocItemHandler),
+    let mut handlers: Vec<TagHandler> = vec![
+        Rc::new(RefCell::new(ParagraphHandler)),
+        Rc::new(RefCell::new(HeadingHandler)),
+        Rc::new(RefCell::new(ListHandler)),
+        Rc::new(RefCell::new(TableHandler::new())),
+        Rc::new(RefCell::new(StyledTextHandler)),
+        Rc::new(RefCell::new(structure::rustdoc::RustdocChromeRemover)),
+        Rc::new(RefCell::new(structure::rustdoc::RustdocHeadingHandler)),
+        Rc::new(RefCell::new(structure::rustdoc::RustdocCodeHandler)),
+        Rc::new(RefCell::new(structure::rustdoc::RustdocItemHandler)),
+        item_collector.clone(),
     ];
 
-    let markdown_writer = MarkdownWriter::new();
-    let markdown = markdown_writer
-        .run(&dom.document, handlers)
-        .context("failed to convert rustdoc HTML to Markdown")?;
+    let markdown = convert_html_to_markdown(html, &mut handlers)?;
 
-    Ok(markdown)
+    let items = item_collector
+        .borrow()
+        .items
+        .values()
+        .cloned()
+        .collect::<Vec<_>>();
+
+    Ok((markdown, items))
 }
 
 fn parse_html(mut html: impl Read) -> Result<RcDom> {
@@ -90,6 +88,20 @@ mod tests {
 
     use super::*;
 
+    fn rustdoc_handlers() -> Vec<TagHandler> {
+        vec![
+            Rc::new(RefCell::new(ParagraphHandler)),
+            Rc::new(RefCell::new(HeadingHandler)),
+            Rc::new(RefCell::new(ListHandler)),
+            Rc::new(RefCell::new(TableHandler::new())),
+            Rc::new(RefCell::new(StyledTextHandler)),
+            Rc::new(RefCell::new(structure::rustdoc::RustdocChromeRemover)),
+            Rc::new(RefCell::new(structure::rustdoc::RustdocHeadingHandler)),
+            Rc::new(RefCell::new(structure::rustdoc::RustdocCodeHandler)),
+            Rc::new(RefCell::new(structure::rustdoc::RustdocItemHandler)),
+        ]
+    }
+
     #[test]
     fn test_main_heading_buttons_get_removed() {
         let html = indoc! {r##"
@@ -106,7 +118,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
@@ -126,7 +138,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
@@ -172,7 +184,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
@@ -191,7 +203,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
@@ -233,7 +245,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
@@ -265,7 +277,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
@@ -301,7 +313,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
@@ -355,7 +367,7 @@ mod tests {
         .trim();
 
         assert_eq!(
-            convert_rustdoc_to_markdown(html.as_bytes()).unwrap(),
+            convert_html_to_markdown(html.as_bytes(), &mut rustdoc_handlers()).unwrap(),
             expected
         )
     }
