@@ -25,7 +25,7 @@ use anyhow::{anyhow, Context, Result};
 use async_trait::async_trait;
 use collections::{HashMap, HashSet};
 use futures::Future;
-use gpui::{AppContext, AsyncAppContext, Model, Task};
+use gpui::{AppContext, AsyncAppContext, Model, SharedString, Task};
 pub use highlight_map::HighlightMap;
 use http::HttpClient;
 use lazy_static::lazy_static;
@@ -58,9 +58,7 @@ use std::{
 };
 use syntax_map::{QueryCursorHandle, SyntaxSnapshot};
 use task::RunnableTag;
-pub use task_context::{
-    BasicContextProvider, ContextProvider, ContextProviderWithTasks, RunnableRange,
-};
+pub use task_context::{ContextProvider, RunnableRange};
 use theme::SyntaxTheme;
 use tree_sitter::{self, wasmtime, Query, QueryCursor, WasmStore};
 
@@ -74,7 +72,7 @@ pub use language_registry::{
 pub use lsp::LanguageServerId;
 pub use outline::{Outline, OutlineItem};
 pub use syntax_map::{OwnedSyntaxLayer, SyntaxLayer};
-pub use text::LineEnding;
+pub use text::{AnchorRangeExt, LineEnding};
 pub use tree_sitter::{Node, Parser, Tree, TreeCursor};
 
 /// Initializes the `language` crate.
@@ -537,7 +535,7 @@ async fn try_fetch_server_binary<L: LspAdapter + 'static + Send + Sync + ?Sized>
     binary
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, Default, PartialEq, Eq)]
 pub struct CodeLabel {
     /// The text to display.
     pub text: String,
@@ -612,6 +610,10 @@ pub struct LanguageConfig {
     /// How to soft-wrap long lines of text.
     #[serde(default)]
     pub soft_wrap: Option<SoftWrap>,
+    /// The name of a Prettier parser that will be used for this language when no file path is available.
+    /// If there's a parser name in the language settings, that will be used instead.
+    #[serde(default)]
+    pub prettier_parser_name: Option<String>,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize, Default, JsonSchema)]
@@ -694,9 +696,10 @@ impl Default for LanguageConfig {
             overrides: Default::default(),
             word_characters: Default::default(),
             collapsed_placeholder: Default::default(),
-            hard_tabs: Default::default(),
-            tab_size: Default::default(),
-            soft_wrap: Default::default(),
+            hard_tabs: None,
+            tab_size: None,
+            soft_wrap: None,
+            prettier_parser_name: None,
         }
     }
 }
@@ -884,12 +887,16 @@ struct RedactionConfig {
     pub redaction_capture_ix: u32,
 }
 
+#[derive(Clone, Debug, PartialEq)]
+enum RunnableCapture {
+    Named(SharedString),
+    Run,
+}
+
 struct RunnableConfig {
     pub query: Query,
-    /// A mapping from captures indices to known test tags
-    pub runnable_tags: HashMap<u32, RunnableTag>,
-    /// index of the capture that corresponds to @run
-    pub run_capture_ix: u32,
+    /// A mapping from capture indice to capture kind
+    pub extra_captures: Vec<RunnableCapture>,
 }
 
 struct OverrideConfig {
@@ -1011,23 +1018,21 @@ impl Language {
             .ok_or_else(|| anyhow!("cannot mutate grammar"))?;
 
         let query = Query::new(&grammar.ts_language, source)?;
-        let mut run_capture_index = None;
-        let mut runnable_tags = HashMap::default();
-        for (ix, name) in query.capture_names().iter().enumerate() {
-            if *name == "run" {
-                run_capture_index = Some(ix as u32);
-            } else if !name.starts_with('_') {
-                runnable_tags.insert(ix as u32, RunnableTag(name.to_string().into()));
-            }
+        let mut extra_captures = Vec::with_capacity(query.capture_names().len());
+
+        for name in query.capture_names().iter() {
+            let kind = if *name == "run" {
+                RunnableCapture::Run
+            } else {
+                RunnableCapture::Named(name.to_string().into())
+            };
+            extra_captures.push(kind);
         }
 
-        if let Some(run_capture_ix) = run_capture_index {
-            grammar.runnable_config = Some(RunnableConfig {
-                query,
-                run_capture_ix,
-                runnable_tags,
-            });
-        }
+        grammar.runnable_config = Some(RunnableConfig {
+            extra_captures,
+            query,
+        });
 
         Ok(self)
     }
@@ -1372,6 +1377,10 @@ impl Language {
             language_name => language_name.to_lowercase(),
         }
     }
+
+    pub fn prettier_parser_name(&self) -> Option<&str> {
+        self.config.prettier_parser_name.as_deref()
+    }
 }
 
 impl LanguageScope {
@@ -1531,6 +1540,15 @@ impl CodeLabel {
         }
         result
     }
+
+    pub fn push_str(&mut self, text: &str, highlight: Option<HighlightId>) {
+        let start_ix = self.text.len();
+        self.text.push_str(text);
+        let end_ix = self.text.len();
+        if let Some(highlight) = highlight {
+            self.runs.push((start_ix..end_ix, highlight));
+        }
+    }
 }
 
 impl Ord for LanguageMatcher {
