@@ -5,6 +5,8 @@
 ; Properties
 
 (property_identifier) @property
+(shorthand_property_identifier) @property
+(shorthand_property_identifier_pattern) @property
 
 ; Function and method calls
 
