@@ -147,7 +147,8 @@ pub fn main() {
                     cx,
                 )
             })
-        });
+        })
+        .unwrap();
     });
 }
 
