@@ -54,10 +54,14 @@ pub enum IconDecoration {
 
 #[derive(Default, PartialEq, Copy, Clone)]
 pub enum IconSize {
+    /// 10px
     Indicator,
+    /// 12px
     XSmall,
+    /// 14px
     Small,
     #[default]
+    /// 16px
     Medium,
 }
 
@@ -102,6 +106,7 @@ pub enum IconName {
     Code,
     Collab,
     Command,
+    Context,
     Control,
     Copilot,
     CopilotDisabled,
@@ -166,19 +171,24 @@ pub enum IconName {
     Rerun,
     Return,
     Reveal,
+    RotateCw,
     Save,
     Screen,
     SelectAll,
+    SearchSelection,
     Server,
     Settings,
     Shift,
     Sliders,
     Snip,
     Space,
+    Sparkle,
+    SparkleFilled,
     Spinner,
     Split,
     Star,
     StarFilled,
+    Stop,
     Strikethrough,
     Supermaven,
     SupermavenDisabled,
@@ -226,6 +236,7 @@ impl IconName {
             IconName::Code => "icons/code.svg",
             IconName::Collab => "icons/user_group_16.svg",
             IconName::Command => "icons/command.svg",
+            IconName::Context => "icons/context.svg",
             IconName::Control => "icons/control.svg",
             IconName::Copilot => "icons/copilot.svg",
             IconName::CopilotDisabled => "icons/copilot_disabled.svg",
@@ -290,19 +301,24 @@ impl IconName {
             IconName::ReplyArrowRight => "icons/reply_arrow_right.svg",
             IconName::Rerun => "icons/rerun.svg",
             IconName::Return => "icons/return.svg",
+            IconName::RotateCw => "icons/rotate_cw.svg",
             IconName::Save => "icons/save.svg",
             IconName::Screen => "icons/desktop.svg",
             IconName::SelectAll => "icons/select_all.svg",
+            IconName::SearchSelection => "icons/search_selection.svg",
             IconName::Server => "icons/server.svg",
             IconName::Settings => "icons/file_icons/settings.svg",
             IconName::Shift => "icons/shift.svg",
             IconName::Sliders => "icons/sliders.svg",
             IconName::Snip => "icons/snip.svg",
             IconName::Space => "icons/space.svg",
+            IconName::Sparkle => "icons/sparkle.svg",
+            IconName::SparkleFilled => "icons/sparkle_filled.svg",
             IconName::Spinner => "icons/spinner.svg",
             IconName::Split => "icons/split.svg",
             IconName::Star => "icons/star.svg",
             IconName::StarFilled => "icons/star_filled.svg",
+            IconName::Stop => "icons/stop.svg",
             IconName::Strikethrough => "icons/strikethrough.svg",
             IconName::Supermaven => "icons/supermaven.svg",
             IconName::SupermavenDisabled => "icons/supermaven_disabled.svg",
