@@ -551,8 +551,7 @@ where
             if let Err(err) = self.await {
                 log::error!("{err:?}");
                 if let Ok(prompt) = cx.update(|cx| {
-                    let detail = f(&err, cx)
-                        .unwrap_or_else(|| format!("{err:?}. Please try again.", err = err));
+                    let detail = f(&err, cx).unwrap_or_else(|| format!("{err}. Please try again."));
                     cx.prompt(PromptLevel::Critical, &msg, Some(&detail), &["Ok"])
                 }) {
                     prompt.await.ok();
